/*  File    : S_plot2C.c
 *  Pierre LE BARS 16/12/2009
 */

#define S_FUNCTION_NAME  S_plot2C
#define S_FUNCTION_LEVEL 2


#include "simstruc.h"

#define NOMBRE(S) ssGetSFcnParam(S, 0)



/*====================*
 * S-function methods *
 *====================*/

static void mdlInitializeSizes(SimStruct *S)
{

    ssSetNumSFcnParams(S, 1);  /* Number of expected parameters */
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return;
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 3);

    if (!ssSetNumInputPorts(S, 2)) return;
    ssSetInputPortWidth(S, 0, 4);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortWidth(S, 1, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    
    if (!ssSetNumOutputPorts(S, 1)) return;
     ssSetOutputPortWidth(S, 0, 2);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetOptions(S ,
                    SS_OPTION_EXCEPTION_FREE_CODE |
                    SS_OPTION_USE_TLC_WITH_ACCELERATOR);
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 */

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
    ssSetModelReferenceSampleTimeDefaultInheritance(S);
}


#define MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#if defined(MDL_INITIALIZE_CONDITIONS)
  static void mdlInitializeConditions(SimStruct *S)
  {
    real_T *xD = ssGetRealDiscStates(S);
   
    xD[0]=0.0;
    xD[1]=0.0;
    xD[2]=0.0;
  }
#endif


/* Function: mdlOutputs =======================================================
 * Abstract:
 *    In this function, you compute the outputs of your S-function
 *    block.
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    real_T            *y    = ssGetOutputPortRealSignal(S,0);
    real_T            *xD    = ssGetRealDiscStates(S);
 
    UNUSED_ARG(tid); /* not used in single tasking mode */

    /* y=Cx+Du */
    y[0]=xD[1];
    y[1]=xD[2];
}



#define MDL_UPDATE  /* Change to #undef to remove function */
#if defined(MDL_UPDATE)
  /* Function: mdlUpdate ======================================================
   * Abstract:
   *    This function is called once for every major integration time step.
   *    Discrete states are typically updated here, but this function is useful
   *    for performing any tasks that should only take place once per
   *    integration step.
   */
  static void mdlUpdate(SimStruct *S, int_T tid)
  {
    real_T      *xD = ssGetRealDiscStates(S);   
    InputRealPtrsType   uPtrs = ssGetInputPortRealSignalPtrs(S,1);
    const real_T      *N    = mxGetPr(NOMBRE(S)); 
    real_T      avant;
    real_T      maintenant;
    real_T      valide;
    real_T      compte;
    
    avant=xD[0];
    compte=xD[1];
    valide=xD[2];
    maintenant=*uPtrs[0];
    if (avant < maintenant) {
        valide=1.0;
    }
    if (valide == 1.0) {
        if (compte < *N) {
            compte = compte+1.0;
        }
        else {
            valide = 0.0;
            compte = 0.0;
        }
    }
    xD[0]=maintenant;
    xD[1]=compte;
    xD[2]=valide;
  }
#endif

#define MDL_TERMINATE
/* Function: mdlTerminate =====================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlStart, this is the place to free it.
 */

static void mdlTerminate(SimStruct *S)
{
     UNUSED_ARG(S); /* unused input argument */
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
