/*  File    : Sstop_simul2_synchro.c
 *  Pierre LE BARS 03/04/2008
 */

#define S_FUNCTION_NAME Sstop_simul2_synchro
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"

#define U(element) (*uPtrs[element])  /* Pointer to Input Port0 */

 

/*====================*
 * S-function methods *
 *====================*/

/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, 0);  /* Number of expected parameters */
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 1);

    if (!ssSetNumInputPorts(S, 1)) return;
    ssSetInputPortWidth(S, 0, 3);
    ssSetInputPortDirectFeedThrough(S, 0, 1);

    if (!ssSetNumOutputPorts(S, 1)) return;
    ssSetOutputPortWidth(S, 0, 2);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    /* Take care when specifying exception free code - see sfuntmpl_doc.c */
    /*ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);*/
    /* ssSetOptions(S,
     *              SS_OPTION_WORKS_WITH_CODE_REUSE |
     *              SS_OPTION_EXCEPTION_FREE_CODE |
     *              SS_OPTION_USE_TLC_WITH_ACCELERATOR);*/
    ssSetOptions(S,
                   SS_OPTION_EXCEPTION_FREE_CODE |
                   SS_OPTION_USE_TLC_WITH_ACCELERATOR);
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *   
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
    ssSetModelReferenceSampleTimeDefaultInheritance(S);      
}

#define MDL_INITIALIZE_CONDITIONS
/* Function: mdlInitializeConditions ========================================
 * Abstract:
 *    Initialize both discrete states to one.
 */
static void mdlInitializeConditions(SimStruct *S)
{
    real_T  *xD = ssGetRealDiscStates(S);
    
    *xD=1.0;
}



/* Function: mdlOutputs =======================================================
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    real_T      debut;
    real_T      compte;
    real_T      nombre;
    real_T            *y    = ssGetOutputPortRealSignal(S,0);
    real_T            *xD   = ssGetDiscStates(S);
    InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);
    UNUSED_ARG(tid); /* not used in single tasking mode */
    
    /* y=Cx+Du */
    debut=*uPtrs[1];
    compte=*xD;
    nombre=*uPtrs[0];
     y[0]=*uPtrs[2];
    if (debut==1.0) {
        if (compte<nombre-2.0) {
            y[1]=0.0;
        }
        else {
            y[1]=1.0;
        }
    }
    else {
        y[1]=0.0;
    }
}



#define MDL_UPDATE
/* Function: mdlUpdate ======================================================
 */
static void mdlUpdate(SimStruct *S, int_T tid)
{
    real_T      debut;
    real_T      compte;
    real_T      nombre;
   real_T  *xD = ssGetDiscStates(S);
   InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);
    
    debut=*uPtrs[1];
    compte=*xD;
    nombre=*uPtrs[0];
    if (debut==1.0) {
        if (compte<=nombre) {
            *xD=*xD+1.0;
        }
        else {
            *xD=nombre;
        }
    }
    else {
        *xD=1.0;
    }
}



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    No termination needed, but we are required to have this routine.
 */
static void mdlTerminate(SimStruct *S)
{
    UNUSED_ARG(S); /* unused input argument */
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
