/*  File    : Sstop_simul_synchro.c
 *  Pierre LE BARS 28/05/2008
 */

#define S_FUNCTION_NAME Sstop_simul_synchro
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"

#define U(element) (*uPtrs[element])  /* Pointer to Input Port0 */
#define NOMBRE(S) ssGetSFcnParam(S, 0)
#define PENTE(S) ssGetSFcnParam(S,1)
#define LEVEL(S) ssGetSFcnParam(S,2)

/*====================*
 * S-function methods *
 *====================*/


static void mdlInitializeSizes(SimStruct *S)
{
     ssSetNumSFcnParams(S, 3);  /* Number of expected parameters */
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 3);

    if (!ssSetNumInputPorts(S, 1)) return;
    ssSetInputPortWidth(S, 0, 2);
    ssSetInputPortDirectFeedThrough(S, 0, 1);

    if (!ssSetNumOutputPorts(S, 1)) return;
    ssSetOutputPortWidth(S, 0, 2);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    /* Take care when specifying exception free code - see sfuntmpl_doc.c */
    /*ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);*/
    /* ssSetOptions(S,
     *              SS_OPTION_WORKS_WITH_CODE_REUSE |
     *              SS_OPTION_EXCEPTION_FREE_CODE |
     *              SS_OPTION_USE_TLC_WITH_ACCELERATOR);*/
    ssSetOptions(S,
                   SS_OPTION_EXCEPTION_FREE_CODE |
                   SS_OPTION_USE_TLC_WITH_ACCELERATOR);
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *   
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
    ssSetModelReferenceSampleTimeDefaultInheritance(S);      
}

#define MDL_INITIALIZE_CONDITIONS
/* Function: mdlInitializeConditions ========================================
 * Abstract:
 *    Initialize both discrete states to one.
 */
static void mdlInitializeConditions(SimStruct *S)
{
    real_T  *xD = ssGetRealDiscStates(S);
    
    xD[0]=1.0;
    xD[1]=0.0;
    xD[2]=0.0;
}



/* Function: mdlOutputs =======================================================
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    real_T            *y    = ssGetOutputPortRealSignal(S,0);
    real_T            *xD   = ssGetDiscStates(S);
    InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);
    const real_T      *N    = mxGetPr(NOMBRE(S)); 
    const real_T      *sens    = mxGetPr(PENTE(S)); 
    const real_T      *niveau    = mxGetPr(LEVEL(S)); 
    boolean_T           test;
    UNUSED_ARG(tid); /* not used in single tasking mode */
    
     y[0]=*uPtrs[0];
     if (xD[0]<*N-1.0) {
            y[1]=0.0;
     }
     else {
            y[1]=1.0;
     }
}      
             

#define MDL_UPDATE
/* Function: mdlUpdate ======================================================
 */
static void mdlUpdate(SimStruct *S, int_T tid)
{
   real_T  *xD = ssGetDiscStates(S);
   InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);
   const real_T      *N    = mxGetPr(NOMBRE(S)); 
    const real_T       *sens    = mxGetPr(PENTE(S)); 
    const real_T       *niveau    = mxGetPr(LEVEL(S)); 
    boolean_T           test;
   
    if (*uPtrs[1]==1.0) {
        if (xD[0]<=*N) {
            if (xD[2]==1.0) {
                xD[0]=xD[0]+1.0;
                xD[2]=1.0;
            }
            else {
                test=((*sens)*(*niveau-xD[1])>=0.0) && ((*sens)*(*uPtrs[0]-*niveau)>=0.0);
                if (test) {
                    xD[0]=xD[0]+1.0;
                    xD[2]=1.0;
                }
                else {
                    xD[0]=1.0;
                    xD[2]=0.0;
                }
            }
        }
         else {
             xD[0]=*N;
             xD[2]=0.0;
         }
    }
    else {
        xD[0]=1.0;
        xD[2]=0.0;
    }
    xD[1]=*uPtrs[0];
}



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    No termination needed, but we are required to have this routine.
 */
static void mdlTerminate(SimStruct *S)
{
    UNUSED_ARG(S); /* unused input argument */
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
