/*  File    : Ssynchro1.c
 *  Pierre LE BARS 03/04/2008
 */

#define S_FUNCTION_NAME Ssynchro
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"

#define U(element) (*uPtrs[element])  /* Pointer to Input Port0 */

 

/*====================*
 * S-function methods *
 *====================*/

/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, 0);  /* Number of expected parameters */
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 3);

    if (!ssSetNumInputPorts(S, 1)) return;
    ssSetInputPortWidth(S, 0, 5);
    ssSetInputPortDirectFeedThrough(S, 0, 1);

    if (!ssSetNumOutputPorts(S, 1)) return;
    ssSetOutputPortWidth(S, 0, 2);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    /* Take care when specifying exception free code - see sfuntmpl_doc.c */
    /*ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);*/
     ssSetOptions(S,
                 SS_OPTION_EXCEPTION_FREE_CODE |
                 SS_OPTION_USE_TLC_WITH_ACCELERATOR);
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *   
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
    ssSetModelReferenceSampleTimeDefaultInheritance(S);      
}

#define MDL_INITIALIZE_CONDITIONS
/* Function: mdlInitializeConditions ========================================
 * Abstract:
 *    Initialize both discrete states to one.
 *
 *    pour dterminer le sens de variation, il faut mmoriser la valeur prcdente de l'entre  
 */
static void mdlInitializeConditions(SimStruct *S)
{
    real_T  *xD = ssGetRealDiscStates(S);
    
    xD[0]=0.0;
    xD[1]=0.0;
    xD[2]=0.0;
}



/* Function: mdlOutputs =======================================================
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    real_T               nombre;
    real_T               niveau;
    real_T               sens;
    real_T               valide;
    real_T               xn;
    real_T               xn_1;
    real_T               compte;
    real_T               synchro;
    real_T              *y    = ssGetOutputPortRealSignal(S,0);
    real_T              *xD   = ssGetDiscStates(S);
    InputRealPtrsType   uPtrs = ssGetInputPortRealSignalPtrs(S,0);
    UNUSED_ARG(tid); /* not used in single tasking mode */
    
    nombre=*uPtrs[0];
    niveau=*uPtrs[1];
    sens=*uPtrs[2];
    xn=*uPtrs[3];
    valide=*uPtrs[4];
    compte=xD[0];
    xn_1=xD[1];
    synchro=xD[2];
    y[0]=xn;
    if (synchro==1.0) {
        if (compte<nombre) {
            y[1]=0.0;
        }
    }
    else {
        y[1]=1.0;
    }
}
    


#define MDL_UPDATE
/* Function: mdlUpdate ======================================================
 */
static void mdlUpdate(SimStruct *S, int_T tid)
{
   boolean_T            test; 
   real_T               nombre;
   real_T               niveau;
   real_T               sens;
   real_T               valide;
   real_T               xn;
   real_T               xn_1;
   real_T               compte;
   real_T               synchro;
   real_T               *xD = ssGetDiscStates(S);
   InputRealPtrsType    uPtrs = ssGetInputPortRealSignalPtrs(S,0);
    
   nombre=*uPtrs[0];
   niveau=*uPtrs[1];
   sens=*uPtrs[2];
   xn=*uPtrs[3];
   valide=*uPtrs[4];
   compte=xD[0];
   xn_1=xD[1];
   synchro=xD[2];
   xD[1]=xn;
   if (synchro==1.0) {
        if (compte<nombre) {
            xD[0]=compte+1.0;;
            xD[2]=synchro;
        }
    }
    else {
        test=(sens*(niveau-xn_1)>0.0)&&(sens*(xn-niveau)>=0.0)&&(valide==1.0);
        if (test) {
            xD[0]=compte+1.0;
            xD[2]=1.0;
        }
        else {
            xD[0]=0.0;
            xD[2]=0.0;
        }
    }
}



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    No termination needed, but we are required to have this routine.
 */
static void mdlTerminate(SimStruct *S)
{
    UNUSED_ARG(S); /* unused input argument */
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
